package robocop;
/**
 * Entschluesselt die Sever-Messages
 * vom Typ "sense_body"
 *
 * @author <A HREF="mailto:miebert@uni-mannheim.de">Michael Ebert</A>
 * @author <A HREF="mailto:jgruentj@uni-mannheim.de">Jens Gruentjes</A>
 * @version 1.13
 */

import java.util.StringTokenizer;

public class BodyEvent extends MsgEvent
{
	double[] data = null;

	/**
	 * Instanziierung eines Objekts vom Typ BodyEvent,
	 * abgeleitet aus der Klasse MsgEvent
	 * @param message "sense_body"-Nachricht, die vom Server geschickt wird
	 * @see MsgEvent
	 */

	public BodyEvent(String message)
	{
		super(message);
		time = evalTime(message);
		data = bodyTokenizer(message);
	}

	/**
	 * bodyTokenizer zerhackt die message in einzelne Stuecke (Tokens),
	 * von denen die relevanten Informationen zurckgegeben werden.
	 * @param message "sense_body"-Nachricht vom Server
	 * @return Feld mit einzelnen Komponenten der Server-Nachricht
	 */

	double[] bodyTokenizer (String message)
	{
		StringTokenizer zerhacker = new StringTokenizer(message, " ");
		double[] ergebnisFeld = new double[BODYSENSE_LENGTH];
		int count = 0;


		// solange bis alle Tokens verarbeitet
		while(zerhacker.hasMoreTokens())
		{
			String tokenAktuell = zerhacker.nextToken();

			//wird nicht benoetigt
			if (tokenAktuell.startsWith("("))
			{}
			else
			{   //ueberfluessige Characters werden entfernt
				tokenAktuell = tokenAktuell.replace(')', ' ');

				// string ==> double
				try
				{
					ergebnisFeld[count] = Double.parseDouble(tokenAktuell);
				}

				catch (NumberFormatException nfe)
				{
					if (tokenAktuell.equals("high"))
						ergebnisFeld[count] = HIGH;
					else if (tokenAktuell.equals("low"))
						ergebnisFeld[count] = LOW;
					else if (tokenAktuell.equals("narrow"))
						ergebnisFeld[count] = NARROW;
					else if (tokenAktuell.equals("normal"))
						ergebnisFeld[count] = NORMAL;
					else if (tokenAktuell.equals("wide"))
						ergebnisFeld[count] = WIDE;
				}
				count++;
			}

		}
		return ergebnisFeld;

	}
}
