package robocop;
/**
 * <p>GUI</p>
 * <p>Beschreibung: visualisiert die berechneten Koordinaten der wahrgenommenen Objekte</p>
 * @author Michael Ebert
 * @author Jens Gruentjes
 * @author Christian Seifried
 * @version 1.7
 */

import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class GUI extends Frame implements Konstanten
{
	private Vector flagList = null;
	private Vector lineList = null;
	private Vector fplayerList = null;
	private Vector eplayerList = null;
	private VisualObj ball = null;
	private VisualObj self = null;
	private VisualObj enemyGoal = null;
	private double areaX1 = 0;
	private double areaY1 = 0;
	private double areaX2 = 0;
	private double areaY2 = 0;
	private double moveX = 0;
	private double moveY = 0;
	private double deltaX = 0;
	private double deltaY = 0;
	private int serverPlayerNr;
	private int playerNr;
	private String teamName;
	private int halbzeit = 1;

	boolean init = true;

	/**
	* Baut die GUI fr den gewnschten Spieler in separatem Fenster auf
	* @param playerNumber Nummer des Spielers, fr den die GUI konstruiert werden soll
	* @param teamName Name des Teams, in dem der Spieler spielt
	*/

	//Konstruktor
	public GUI(int playerNumber, String teamName)
	{
		this.playerNr = playerNumber;
		this.teamName = teamName;
		addWindowListener(new WindowClosingAdapter(false));
		setSize (480,362);
		setResizable(false);
		setVisible(true);
		setBackground(new Color(0,200,0));
	}

	//wird von Frame aufgerufen, wenn sich am zust. des fensters was ndert
	/**
	* aus Klasse Container berschriebene Methode, die Linien, Flags und alle anderen Objekte zeichnet
	* @param g
	*/
	public void paint(Graphics g)
	{
		g.translate(240,186);
		showLines(g);
		showFlags(g);
		showObjects(g);
	}

	private void showObjects(Graphics g1)
	{
		if((ball != null) && (ball.getTime() > 3000))
		{
			halbzeit = -1;
		}

		//gegnertor
		if (enemyGoal != null)
		{
			g1.setColor(Color.red);
			g1.fillOval((int)(halbzeit*enemyGoal.getXpos()*4)-2, (int)(halbzeit*enemyGoal.getYpos()*4)-2,5,5);
		}
		//gegner
		if (eplayerList != null)
		{
			g1.setColor(Color.blue);
			for (Enumeration e = eplayerList.elements() ; e.hasMoreElements() ;)
			{
				VisualObj enemy = (VisualObj) e.nextElement();
				g1.fillOval((int)(halbzeit*enemy.getXpos()*4)-2,(int)(halbzeit*enemy.getYpos()*4)-2,5,5);
			}
		}
		//eigene
		if (fplayerList != null)
		{
			g1.setColor(Color.yellow);
			for (Enumeration e = fplayerList.elements() ; e.hasMoreElements() ;)
			{
				VisualObj enemy = (VisualObj) e.nextElement();
				g1.fillOval((int) (halbzeit*enemy.getXpos()*4)-2,(int) (halbzeit*enemy.getYpos()*4)-2,5,5);
			}
		}



		//mich selbst und vektor
		if (self != null)
		{
			g1.setColor(Color.magenta);
			g1.fillOval((int) (halbzeit*self.getXpos()*4)-2, (int) (halbzeit*self.getYpos()*4)-2,6,6);
			g1.drawLine((int) (halbzeit*self.getXpos()*4)-2, (int) (halbzeit*self.getYpos()*4)-2, (int) (halbzeit*self.getXpos()*4+halbzeit*self.getDeltaX()*10)-2, (int) (halbzeit*self.getYpos()*4+halbzeit*self.getDeltaY()*10)-2);
		}

		// Move-Position
		g1.setColor(Color.cyan);
		g1.fillOval((int) (halbzeit*moveX*4)-2, (int) (halbzeit*moveY*4)-2,6,6);

		//ball und vektor
		if (ball != null)
		{
			g1.setColor(Color.white);
			g1.fillOval((int) (halbzeit*ball.getXpos()*4)-2, (int) (halbzeit*ball.getYpos()*4)-2,6,6);
			g1.drawLine((int) (halbzeit*ball.getXpos()*4)-2, (int) (halbzeit*ball.getYpos()*4)-2, (int) (halbzeit*ball.getXpos()*4+halbzeit*ball.getDeltaX()*10)-2, (int) (halbzeit*ball.getYpos()*4+halbzeit*ball.getDeltaY()*10)-2);
		}
	}

	private void showFlags(Graphics g1)
	{
		//zeichnen aller flaggen
		g1.setColor(Color.black);
		for (int i=0; i<55; i++)
		{
			g1.fillRect((int) (halbzeit*FLAGSXY[i][0]*4)-2,(int) (halbzeit*FLAGSXY[i][1]*4)-2,5,5);
		}

		//zeichnen der gesehenen flaggen
		g1.setColor(Color.red);
		if (flagList != null)
		{
			for (Enumeration e = flagList.elements() ; e.hasMoreElements() ;)
			{
				VisualObj flag = (VisualObj) e.nextElement();
				g1.fillRect((int) (halbzeit*flag.getXpos()*4)-2,(int) (halbzeit*flag.getYpos()*4)-2,5,5);
			}
		}
	}//ende showFlags

	private void showLines(Graphics g1)
	{
		//Mittellinie
		g1.setColor(Color.black);
		g1.drawLine((int)(halbzeit*FLAGSXY[12][0]*4),(int)(halbzeit*FLAGSXY[12][1]*4),(int)(halbzeit*FLAGSXY[42][0]*4),(int)(halbzeit*FLAGSXY[42][1]*4));
		//Spielfeld
		g1.drawLine((int)(halbzeit*FLAGSXY[11][0]*4),(int)(halbzeit*FLAGSXY[11][1]*4),(int)(halbzeit*FLAGSXY[13][0]*4),(int)(halbzeit*FLAGSXY[13][1]*4));
		g1.drawLine((int)(halbzeit*FLAGSXY[41][0]*4),(int)(halbzeit*FLAGSXY[41][1]*4),(int)(halbzeit*FLAGSXY[43][0]*4),(int)(halbzeit*FLAGSXY[43][1]*4));
		g1.drawLine((int)(halbzeit*FLAGSXY[11][0]*4),(int)(halbzeit*FLAGSXY[11][1]*4),(int)(halbzeit*FLAGSXY[41][0]*4),(int)(halbzeit*FLAGSXY[41][1]*4));
		g1.drawLine((int)(halbzeit*FLAGSXY[13][0]*4),(int)(halbzeit*FLAGSXY[13][1]*4),(int)(halbzeit*FLAGSXY[43][0]*4),(int)(halbzeit*FLAGSXY[43][1]*4));


		//gesehene Linien
		if (lineList != null)
		{
		g1.setColor(Color.red);
		for(Enumeration e = lineList.elements() ; e.hasMoreElements();)
		{
			VisualObj line = (VisualObj) e.nextElement();
			switch(line.getCode().charAt(2))
			{
				case 't':
					g1.drawLine((int)(halbzeit*FLAGSXY[11][0]*4),(int)(halbzeit*FLAGSXY[11][1]*4),(int)(halbzeit*FLAGSXY[13][0]*4),(int)(halbzeit*FLAGSXY[13][1]*4));
					break;
				case 'b':
					g1.drawLine((int)(halbzeit*FLAGSXY[41][0]*4),(int)(halbzeit*FLAGSXY[41][1]*4),(int)(halbzeit*FLAGSXY[43][0]*4),(int)(halbzeit*FLAGSXY[43][1]*4));
					break;
				case 'l':
					g1.drawLine((int)(halbzeit*FLAGSXY[11][0]*4),(int)(halbzeit*FLAGSXY[11][1]*4),(int)(halbzeit*FLAGSXY[41][0]*4),(int)(halbzeit*FLAGSXY[41][1]*4));
					break;
				case 'r':
					g1.drawLine((int)(halbzeit*FLAGSXY[13][0]*4),(int)(halbzeit*FLAGSXY[13][1]*4),(int)(halbzeit*FLAGSXY[43][0]*4),(int)(halbzeit*FLAGSXY[43][1]*4));
					break;
				default:
					break;
			}//ende switch
		}//ende for
		// Aktionsbereich des Spielers
		g1.setColor(Color.magenta);
		g1.drawLine((int)(halbzeit*areaX1*4),(int)(halbzeit*areaY1*4),(int)(halbzeit*areaX1*4),(int)(halbzeit*areaY2*4));
		g1.drawLine((int)(halbzeit*areaX1*4),(int)(halbzeit*areaY1*4),(int)(halbzeit*areaX2*4),(int)(halbzeit*areaY1*4));
		g1.drawLine((int)(halbzeit*areaX2*4),(int)(halbzeit*areaY2*4),(int)(halbzeit*areaX1*4),(int)(halbzeit*areaY2*4));
		g1.drawLine((int)(halbzeit*areaX2*4),(int)(halbzeit*areaY2*4),(int)(halbzeit*areaX2*4),(int)(halbzeit*areaY1*4));
	}//ende if
	}//ende showLines

	/**
	* wird laufend aufgerufen, wenn vom Server neue Nachrichten kommen. Spielfeld wird dann mit den aktuellen Objekten neu gezeichnet
	* @param serverPlayerNr vom Server zugewiesene Nummer des Spielers, fr den die GUI angezeigt wird
	* @param FlagList Liste der gesehenen Flags
	* @param LineList Liste der gesehenen Linien
	* @param FriendlyPlayer Liste der gesehenen Mitspieler
	* @param EnemyPlayerList Liste der gesehenen Gegenspieler
	* @param self sich selbst
	* @param ball der Ball
	* @param enemyGoal das gegnerische Tor
	*/
	public void updateGUI(int serverPlayerNr, Vector FlagList, Vector LineList, Vector FriendlyPlayerList, Vector EnemyPlayerList, VisualObj self, VisualObj ball, VisualObj enemyGoal, double areaX1, double areaY1, double areaX2, double areaY2, double moveX, double moveY)
	{
		this.flagList = FlagList;
		this.lineList = LineList;
		this.fplayerList = FriendlyPlayerList;
		this.eplayerList = EnemyPlayerList;
		this.self = self;
		this.ball = ball;
		this.enemyGoal = enemyGoal;
		this.serverPlayerNr = serverPlayerNr;
	this.areaX1 = areaX1;
	this.areaY1 = areaY1;
	this.areaX2 = areaX2;
	this.areaY2 = areaY2;
	this.moveX = moveX;
	this.moveY = moveY;

	setTitle(" Weltbild v. Spieler ("+teamName+") eigene: "+String.valueOf(playerNr)+", server: "+String.valueOf(this.serverPlayerNr));
		repaint();
	}
}//end class GUI

class WindowClosingAdapter extends WindowAdapter
{
	private boolean exitSystem;

	/**
	 * Erzeugt einen WindowClosingAdapter zum Schliessen
	 * des Fensters. Ist exitSystem true, wird das komplette
	 * Programm beendet.
	 */
	public WindowClosingAdapter(boolean exitSystem)
	{
		this.exitSystem = exitSystem;
	}

	/**
	 * Erzeugt einen WindowClosingAdapter zum Schliessen
	 * des Fensters. Das Programm wird nicht beendet.
	 */
	public WindowClosingAdapter()
	{
		this(true);
	}

	/**
	* berschriebene Methode aus Klasse WindowsAdapter,
	* die das Fenster fr die GUI schliesst, wenn das Programm beendet wird
	*/
	public void windowClosing(WindowEvent event)
	{
		event.getWindow().setVisible(false);
		event.getWindow().dispose();
		if (exitSystem)
		{
			System.exit(0);
		}
	}
}//ende class WindowClosingAdapter
