package robocop;
/** Vaterklasse zum Auswerten/Erfassen aller Servernachrichten
 * @author Paul
 * @author Dirk
 */

public class MsgEvent implements Konstanten
{
	/**
	 */	
	protected String message = "";
	/**
	 */	
	protected int time = UNKNOWN;
	/**
	 */	
	protected int type = UNKNOWN;

	/** Konstruktor
	 * @param message die Servermessage
	 */	
	public MsgEvent(String message)
	{
		this.message = message;
		type = evalType(message);
	}

	/** Gibt die Servernachrinht zurueck
	 * @return Servernachricht als String
	 */	
	public String getMessage()	{return message;}
	/** Gibt den Timecode der Nachricht zurueck
	 * @return Timecode der Nachricht  als Integer
	 */	
	public int getTime()	{return time;}
	/** Gibt den Messagetype zurueck
	 * @return MessageType als Integer (siehe Konstanten.java)
	 */	
	public int getType()	{return type;}

	int evalTime(String serverMessage)
	{
		int sp1 = serverMessage.indexOf(" ");
		int sp2 = serverMessage.indexOf(" ", sp1 + 1);
		int res = UNKNOWN;
		try
		{
			res = Integer.parseInt(serverMessage.substring(sp1 + 1, sp2));
		}
		catch(NumberFormatException ne)
		{
			if(debug)
			{
				System.out.println("!EXCEPTION! - MsgEvent - time");
				ne.printStackTrace();
			}
			res = UNKNOWN;
		}
		return res;
	}

	int evalType(String serverMessage)
	{
		String prefix = serverMessage.substring(1,4);

		//Error-Message
		if (serverMessage.length() < 4)
			return ERROR;

		//Error-Message
		else if (prefix.equals("err"))
			return ERROR;

		//Message vom Typ "sense_body"
		else if (prefix.equals("sen"))
			return SENSEBODY;

		//Message vom Typ "see"
		else if (prefix.equals("see"))
			return SEE;

		//Message vom Typ "hear"
		else if (prefix.equals("hea"))
			return HEAR;

		//Message vom Typ "server_param"
		else if (prefix.equals("ser"))
			return SERVERPARAM;

		//Message vom Typ "player_param"
		else if (prefix.equals("pla"))
			return PLAYERPARAM;

		//Message vom Typ "init"
		else if (prefix.equals("ini"))
			return INIT;

		else
		{
			if(debug) System.out.println("\t!Fehler! - MsgEvent - Nachrichtentyp nicht ermittelt!");
			return ERROR;
		}
	} //end evalType

	/** Bestimmt den aktuellen Spielmodus
	 * @param message der Spielmodus als String
	 * @return der Spielmodus als Integer (siehe Konstanten.java)
	 */	
	public int evalPlayMode(String message){
		if (message.equals("play_on"))
			return PLAY_ON;
		else if (message.equals("before_kick_off"))
			return BEFORE_KICK_OFF;
		else if (message.equals("time_over"))
			return TIME_OVER;
		else if (message.equals("kick_off_r"))
			return KICK_OFF_RIGHT;
		else if (message.equals("kick_off_l"))
			return KICK_OFF_LEFT;
		else if (message.equals("kick_in_r"))
			return KICK_IN_RIGHT;
		else if (message.equals("kick_in_l"))
			return KICK_IN_LEFT;
		else if (message.equals("free_kick_r"))
			return FREE_KICK_RIGHT;
		else if (message.equals("free_kick_l"))
			return FREE_KICK_LEFT;
		else if (message.equals("corner_kick_r"))
			return CORNER_KICK_RIGHT;
		else if (message.equals("corner_kick_l"))
			return CORNER_KICK_LEFT;
		else if (message.equals("goal_kick_r"))
			return GOAL_KICK_RIGHT;
		else if (message.equals("goal_kick_l"))
			return GOAL_KICK_LEFT;
		else if (message.equals("drop_ball"))
			return DROP_BALL;
		else if (message.equals("offside_r"))
			return OFFSIDE_RIGHT;
		else if (message.equals("offside_l"))
			return OFFSIDE_LEFT;
		else if (message.equals("foul_r"))
			return FOUL_RIGHT;
		else if (message.equals("foul_l"))
			return FOUL_LEFT;
		else if (message.equals("goalie_catch_ball_r"))
			return GOALIE_CATCH_BALL_RIGHT;
		else if (message.equals("goalie_catch_ball_l"))
			return GOALIE_CATCH_BALL_LEFT;
		else if (message.equals("time_up_without_a_team"))
			return TIME_UP_WITHOUT_A_TEAM;
		else if (message.equals("time_up"))
			return TIME_UP;
		else if (message.equals("halftime"))
			return HALFTIME;
		else if (message.equals("time_extended"))
			return TIME_EXTENDED;
		else if (message.startsWith("goal_r"))
			return GOAL_RIGHT;
		else if (message.startsWith("goal_l"))
			return GOAL_LEFT;
		else
			return UNKNOWN;
	} //end evalPlayMode

}
