package robocop;
/**
 * Diese Klasse ist verantwortl. fuer die Netzwerkkommunikation ber UDP mit
 * dem rcssserver.
 *
 * @author <a href="mailto:jgruentj@uni-mannheim.de">Jens Grntjes</a><br>
 * @author <a href="mailto:miebert@uni-mannheim.de">Michael Ebert</a><br>
 * @author <a href="mailto:christiansei@gmx.de">Christian Seifried</a>
 * @version 1.57
 */

import java.io.*;
import java.net.*;

public class NetTraffic implements Konstanten
{
	private int serverPort = 6000;          //default
	private String serverName = "localhost";  //default
	private InetAddress serverAddr = null;
	private DatagramSocket socket = null;

/**
 * Erzeugt einen Netzwerk-Socket
 * @exception SocketException wird abgefangen, wenn Socket nicht erzeugt werden kann
 * @exception UnknownHostException wird abgefangen, wenn Server-Name nicht ermittelt werden konnte
 */

	public NetTraffic(String serverName, int serverPort)
	{
		this.serverName = serverName;
		this.serverPort = serverPort;

		try	//erzeugt Socket mit lokalem bel. Port
		{
			socket = new DatagramSocket();
			if(debug) System.out.println("STATUS - NetTraffic - LocalPort: " + socket.getPort());
		}
		catch (SocketException se)
		{
			System.out.println("!EXCEPTION! - NetTraffic - Socket konnte nicht erzeugt werden!");
		}

		try
		{
			serverAddr = InetAddress.getByName(serverName);
		}
		catch (UnknownHostException uhe)
		{
			System.out.println("!EXCEPTION! - NetTraffic - Host "+serverAddr+" nicht bekannt!");
		}
	}


/**
 * UDP-Paket wird zum Server gesendet
 * @param message Nachricht, die an Server bermittelt werden soll
 * @exception IOException Paket kann nicht gesendet werden
 */

	void sendePaket(String message)
	{
		if(debug)	System.out.println("STATUS - NetTraffic - Send: " + message);

		//String ==> byte-Feld
		byte[] byteMessage = message.getBytes();

		//UDP-Paket erzeugen
		DatagramPacket paketSende = new DatagramPacket(byteMessage, byteMessage.length, serverAddr, serverPort);

		try
		{
			//der eigentliche Sendevorgang
			socket.send(paketSende);
		}
		catch (IOException ioe)
		{
			System.out.println("!EXCEPTION! - NetTraffic - Paket konnte nicht gesendet werden!");
		}
	}

/**
 * In dieser Funktion wird ein UDP-Paket vom Server empfangen
 * @return entpackte Nachricht vom Server
 * @exception IOException Paket kann nicht empfangen werden
 */

	String empfangePaket()
	{
		byte[] byteEmpfang = new byte[1024];
		DatagramPacket paketEmpfang = new DatagramPacket(byteEmpfang, byteEmpfang.length);

		try
		{
			//der eigentliche Empfangsvorgang
			socket.receive(paketEmpfang);
		}
		catch (IOException ioe)
		{
			System.out.println("!EXCEPTION! - NetTraffic - Paket konnte nicht empfangen werden!");
		}

		//byte ==> String
		String paketInhalt = new String(paketEmpfang.getData());

		//Aktualisierung des Server-Ports, da Client-Messages EINDEUTIG ueber Port
		//Nummer identifiziert werden
		serverPort = paketEmpfang.getPort();

		return paketInhalt.trim();
	}
}