package robocop;

import java.util.*;

/** SeeEvent spezialisiert die MsgEvent Klasse zum Auswerten der Seenachrichten vom Server
 * @author Paul
 * @author Dirk
 * @see MsgEvent
 */
public class SeeEvent extends MsgEvent
{
		private Vector fPlayerList = new Vector();	// eigene mannschaft
		private Vector ePlayerList = new Vector();	// gegner
		private String myTeamname = "";
		private Vector flagList = new Vector();
		private Vector lineList = new Vector();
		private VisualObj ball = null;
		private VisualObj goalR = null;
		private VisualObj goalL = null;

		/** Konstruktor
		 * @param message "See-Nachricht"
		 * @param myTeamname eigener Team-Name
		 * @exception NumberFormatException wenn Distanz oder Winkel keine Zahl ist
		 */
	public SeeEvent(String message, String myTeamname)
		{
				super(message);
				time = evalTime(message);
				this.myTeamname = myTeamname;
				// use parantheses as string delimiters
				StringTokenizer	tokenizer = new StringTokenizer(message, "()");
				// skip timecode
				if (tokenizer.hasMoreTokens()) tokenizer.nextToken();


				// process all seen objects
				while (tokenizer.hasMoreTokens())
				{
					// get the object name
					// in the example the next Token would be:
					// "g r" for the right goal
					String code = tokenizer.nextToken();
					double distance = OUTBOUNDS;
					double angle = OUTBOUNDS;

					// sometimes see messages arrive without any objects following
					if (!code.equals(" "))
					{

						// get distance and angle
						String data = (tokenizer.hasMoreTokens()) ? (tokenizer.nextToken()) : ("");

						StringTokenizer st   = new StringTokenizer(data, " ");
						try
						{
							distance  = (st.hasMoreTokens()) ? (Double.parseDouble(st.nextToken())) : OUTBOUNDS;
						}
						catch(NumberFormatException ne)
						{
							if(debug)
							{
								System.out.println("!EXCEPTION! - SeeEvent - distance");
								ne.printStackTrace();
							}
							distance = OUTBOUNDS;
						}
						try
						{
							angle = (st.hasMoreTokens()) ? (Double.parseDouble(st.nextToken())) : OUTBOUNDS;
						}
						catch(NumberFormatException ne)
						{
							if(debug)
							{
								System.out.println("!EXCEPTION! - SeeEvent - angle");
								ne.printStackTrace();
							}
							angle = OUTBOUNDS;
						}

						if((distance != OUTBOUNDS) && (angle != OUTBOUNDS))
						{
							// create a new object info
							VisualObj myVisObj = new VisualObj(this.getTime(), code, distance, angle);

							switch (code.charAt(0)) {
								case 'b':
									ball = myVisObj;
									break;
								case 'g':
									if(code.charAt(2) == 'r')
									{
										 goalR = myVisObj;
										 goalR.setXpos(FLAGSXY[29][0]/**SuperBrain.ourSide*/);
										 goalR.setYpos(FLAGSXY[29][1]/**SuperBrain.ourSide*/);
									}
									else
									{
										 goalL = myVisObj;
										 goalL.setXpos(FLAGSXY[25][0]/**SuperBrain.ourSide*/);
										 goalL.setYpos(FLAGSXY[25][1]/**SuperBrain.ourSide*/);
									}
									break;
								case 'p':
									if (isFriend(code) == 1) {fPlayerList.add(myVisObj);
									} else if (isFriend(code) == 2){ePlayerList.add(myVisObj);}
									//else {hier eventuell mitspieler, die zuweit weg sind noch einkalk.?}
									break;
								case 'f':
									int flagNummer = evalFlag(code);
									if(flagNummer != UNKNOWN)
									{
										myVisObj.setXpos(FLAGSXY[flagNummer][0]);
										myVisObj.setYpos(FLAGSXY[flagNummer][1]);
										flagList.add(myVisObj);
									}
									break;
								case 'l':
									lineList.add(myVisObj);
									break;
							}//Ende switch
						}//Ende if
						// skip to next object
						if(tokenizer.hasMoreTokens() & (distance != OUTBOUNDS) & (angle != OUTBOUNDS)) tokenizer.nextToken();
					}//Ende if
				}//Ende while
		}//Ende SeeEvent-Konstruktor

		/** Gibt den mitspielervector zurueck
		 * @return Liste der eigenen Mitspieler als Vector
		 */
		public Vector getFPlayerList()
		{
				return fPlayerList;
		}

		/** Gibt den gegnervector zurueck
		 * @return Liste der gegnerischen Spieler als Vector
		 */
		public Vector getEPlayerList()
		{
				return ePlayerList;
		}

		 /**
		* prft, ob gesehener Spieler Freund oder Feind ist
		* @param code Teil der "See-Nachricht", die Informationen ber gesehene Spieler enthlt
		* @return 0-eigener Mitspieler und nicht erreichbar, 1-eigener Mitspieler und erreichbar, 2-Gegner
		*/
		private int isFriend(String code){
			boolean seeNumber = false;
			boolean seeTeam = false;
			int status = 2;
			String temp = "";

			StringTokenizer str   = new StringTokenizer(code," ");
			//let out "p"
			str.nextToken();
			while(str.hasMoreTokens()){
				if (seeTeam){
					return 1;
				}else{
					seeTeam = true;
					//get the team-name
					temp = str.nextToken();
					temp = temp.substring(1,temp.length()-1);
					if (!temp.equalsIgnoreCase(myTeamname)){ return status;
					} else {status = 0;}
				}
			}
		return status;
		}

	 /**
		* Liefert den Gegen-/MitSpieler, der zum bergebenen Objekt am nchsten ist.
		* Wenn keiner nher ist als ich wird self zurckgegeben
		*
		* @param self Das visuelle Objekt, dass mich selbst abbildet
		* @param obj Das visuelle Objekt, zu dem wir den nchsten suchen
		* @param Players Die Liste der verfgbaren (sichtb.) Spielern
		* @return VisualObj
		*/

		public VisualObj getNexttoObj(VisualObj self, VisualObj obj, Vector Players){
			VisualObj player = null;
			VisualObj retplayer = self;
			double nTemp = 0.0;

			//get our distance to the object
			double deltaX = obj.getXpos() - self.getXpos();
			double deltaY = obj.getYpos() - self.getYpos();
			double distance = Math.sqrt(deltaX * deltaX + deltaY * deltaY);

			for (int i=0;i<Players.size();i++){
				player = (VisualObj) Players.elementAt(i);
				deltaX = obj.getXpos() - player.getXpos();
				deltaY = obj.getYpos() - player.getYpos();
				nTemp = Math.sqrt(deltaX * deltaX + deltaY * deltaY);

				if( nTemp < distance ){
					distance = nTemp;
					retplayer = player;
				}

			}
			return retplayer;
		}//end getNexttoGoal



		/** Gibt den Flaggenvector zurueck
		 * @return Liste als Vector
		 */
		public Vector getFlagList()
		{
				return flagList;
		}

		/** Gibt den Linienvector zurueck
		 * @return Liste als Vector
		 */
		public Vector getLineList()
		{
				return lineList;
		}

		/** Gibt das BallObject zureuck
		 * @return Ball als VisualObj
		 */
		public VisualObj getBall()
		{
				return ball;
		}

		/** Gibt das Rechte Tor zurueck
		 * @return rechtes Tor als VisuakObj
		 */
		public VisualObj getGoalR()
		{
				return goalR;
		}

		/** Gibt das linke Tor zurueck
		 * @return linkes Tor als VisualObj
		 */
		public VisualObj getGoalL()
		{
				return goalL;
		}

		/** Weist dem FlaggeCode eine Nummer(siehe Konsanten.java) zu
		 * @param code Teil der "See-Nachricht", der Informationen ber die Flags enthlt
		 * @return Flag-Nummer als Integer (siehe Konstanten.java)
		 */
		public int evalFlag(String code)
		{
			if (code.startsWith("t l 50", 2))
				return 0;
			else if (code.startsWith("t l 40", 2))
				return 1;
			else if (code.startsWith("t l 30", 2))
				return 2;
			else if (code.startsWith("t l 20", 2))
				return 3;
			else if (code.startsWith("t l 10", 2))
				return 4;
			else if (code.startsWith("t 0", 2))
				return 5;
			else if (code.startsWith("t r 10", 2))
				return 6;
			else if (code.startsWith("t r 20", 2))
				return 7;
			else if (code.startsWith("t r 30", 2))
				return 8;
			else if (code.startsWith("t r 40", 2))
				return 9;
			else if (code.startsWith("t r 50", 2))
				return 10;
			else if (code.equals("f l t"))
				return 11;
			else if (code.equals("f c t"))
				return 12;
			else if (code.equals("f r t"))
				return 13;
			else if (code.startsWith("l t 30", 2))
				return 14;
			else if (code.startsWith("r t 30", 2))
				return 15;
			else if (code.startsWith("l t 20", 2))
				return 16;
			else if (code.startsWith("p l t", 2))
				return 17;
			else if (code.startsWith("p r t", 2))
				return 18;
			else if (code.startsWith("r t 20", 2))
				return 19;
			else if (code.startsWith("l t 10", 2))
				return 20;
			else if (code.startsWith("r t 10", 2))
				return 21;
			else if (code.startsWith("f g l t", 2))
				return 22;
			else if (code.startsWith("f g r t", 2))
				return 23;
			else if (code.startsWith("l 0", 2))
				return 24;
			//25 fehlt, da linkes Tor
			else if (code.startsWith("p l c", 2))
				return 26;
			else if (code.equals("f c"))
				return 27;
			else if (code.startsWith("p r c", 2))
				return 28;
			//29 fehlt, da rechtes Tor
			else if (code.startsWith("r 0", 2))
				return 30;
			else if (code.startsWith("f g l b", 2))
				return 31;
			else if (code.startsWith("f g r b", 2))
				return 32;
			else if (code.startsWith("l b 10", 2))
				return 33;
			else if (code.startsWith("r b 10", 2))
				return 34;
			else if (code.startsWith("l b 20", 2))
				return 35;
			else if (code.startsWith("p l b", 2))
				return 36;
			else if (code.startsWith("p r b", 2))
				return 37;
			else if (code.startsWith("r b 20", 2))
				return 38;
			else if (code.startsWith("l b 30", 2))
				return 39;
			else if (code.startsWith("r b 30", 2))
				return 40;
			else if (code.equals("f l b"))
				return 41;
			else if (code.equals("f c b"))
				return 42;
			else if (code.equals("f r b"))
				return 43;
			else if (code.startsWith("b l 50", 2))
				return 44;
			else if (code.startsWith("b l 40", 2))
				return 45;
			else if (code.startsWith("b l 30", 2))
				return 46;
			else if (code.startsWith("b l 20", 2))
				return 47;
			else if (code.startsWith("b l 10", 2))
				return 48;
			else if (code.startsWith("b 0", 2))
				return 49;
			else if (code.startsWith("b r 10", 2))
				return 50;
			else if (code.startsWith("b r 20", 2))
				return 51;
			else if (code.startsWith("b r 30", 2))
				return 52;
			else if (code.startsWith("b r 40", 2))
				return 53;
			else if (code.startsWith("b r 50", 2))
				return 54;
			else return UNKNOWN;
		}
}
