package robocop;
/**
 * <p>SuperBody</p>
 * <p>Beschreibung: Bewegungsapparat des Clients, fhrt die von der KI veranlassten
 * Bewegungen aus bzw. sendet dies Befehle an den Server
 * </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Uni Mannheim</p>
 * @author Christian Seifried
 * @version 1.0
 */

public class SuperBody implements Konstanten {

	private NetTraffic netTraffic;

	/**
	 * Konstruktor
	 * @param netTraffic Klasse die Netzwerkkommunikation ermglicht
	 */
	public SuperBody(NetTraffic netTraffic) {
		this.netTraffic = netTraffic;
	}

	/**
	 * Sendet das Paket, um den Torwart, innerhalb des Strafr. den Ball fangen zu lassen
	 * details im servermanual unter 4.2.-> client control
	 * @param richtung Winkel zum Ball
	 */

	public void gCatch(double richtung){
		netTraffic.sendePaket(new String("(catch " + richtung + ")"));
	}

	/**
	 * Sendet das Paket, um die Einstellung des Blickfeldes zu ndern
	 * @param breite
	 * @param qualitaet
	 * <p>details im rcssmanual auf seite 23</p>
	 */

	public void chView(int breite, int qualitaet){
		String sBreite, sQualitaet;

		if (breite == NORMAL) sBreite = "normal";
		else if (breite == NARROW) sBreite = "narrow";
		else sBreite = "wide";

		if (qualitaet == HIGH) sQualitaet = "high";
		else sQualitaet = "low";

		netTraffic.sendePaket(new String("(change_view " +sBreite+ " " +sQualitaet+ ")"));
	}

	/**
	 * Sendet das Packet, um den Cleint zum laufen zu veranlassen
	 * @param power Geschwindigkeit mit der gerannt wird, beachte Stammina
	 * <br>ACHTUNG - </br> neg. power (rckwrts) verbraucht doppelt Stammina
	 */

	public void dash(double power){
		netTraffic.sendePaket(new String("(dash " +power+ ")"));
	}

	/**
	 * Sendet das Packet, um den Client zum Laufen zu veranlassen
	 * @param power Geschwindigkeit mit der gekickt wird, beachte Stammina
	 * @param direction Richtung in die gekickt wird
	 */

	public void kick(double power, double direction){
		netTraffic.sendePaket(new String("(kick " +power+ " " +direction+ ")"));
	}

	/**
	 * Sendet das Packet, um den Client auf die angegebene Pos zu stellen
	 * @param x X-Kooridante
	 * @param y Y-Kooridante
	 * <p>beachte: nur mglich whrend before_kick_off</p>
	 */

	public void moveTo(double x, double y){
		netTraffic.sendePaket(new String("(move " +x+ " " +y+ ")"));
	}

	/**
	 * Sendet das Paket, um den Client eine Message rufen zu lassen
	 * @param message Zu rufende Botschaft
	 */

	public void say(String message){
		netTraffic.sendePaket(new String("(say " +message+ ")"));
	}

	/**
	 * Sendet das Paket, das den server veranlasst unseren Zustand zu senden
	 */

	public void reqtSensebody(){
		netTraffic.sendePaket(new String("(sense_body)"));
	}

	/**
	 * Sendet das Paket, das den server veranlasst den Spielstand zu senden
	 */

	public void reqScore(){
		netTraffic.sendePaket(new String("(score)"));
	}

	/**
	 * Sendet das Paket, um den Client zu drehen
	 *
	 * @param winkel Winkel, um den sich gedreht werden soll
	 * <p>Winkel muss zw. min- und maxmoment liegen!</p>
	 */

	public void turn(double winkel){
		netTraffic.sendePaket(new String("(turn " +winkel+ ")"));
	}

	/**
	 * Sendet das Paket, um den Blickwinkel des Spielers zu drehen
	 *
	 * @param winkel Winkel, um den sich gedreht werden soll
	 * <p>winkel muss zw. min- und maxneckmoment liegen</p>
	 */

	public void turnNeck(int winkel){
		netTraffic.sendePaket(new String("(turn_neck " +winkel+ ")"));
	}

}//end class