package robocop;

import java.util.*;
import java.math.*;

/** Vaterklasse der KI eines Spielers mit minimaler Strategie
 * und dem Weltmodell
 * @author Paul
 * @author Dirk
 */

public class TheBrain implements Konstanten
{
	public boolean showGUI = false;
	public boolean showCMD = false;

	protected NetTraffic netTraffic = null;
	protected SuperBody body = null;
	protected GUI gui = null;
	protected SeeEvent currSee = null;
	protected BodyEvent currFeel = null;
	protected HearEvent currHear = null;

	protected String teamName = null;
	protected int ourSide = LEFTSIDE;
	protected int status = PLAYING;
	protected int playerNumber = UNKNOWN;
	protected int serverPlayerNr = UNKNOWN;

	protected double initX = OUTBOUNDS;
	protected double initY = OUTBOUNDS;
	protected double playX = OUTBOUNDS;
	protected double playY = OUTBOUNDS;
	protected double minX = OUTBOUNDS;
	protected double maxX = OUTBOUNDS;
	protected double minY = OUTBOUNDS;
	protected double maxY = OUTBOUNDS;
	protected double ballOrigin = OUTBOUNDS;
	protected int lastKnownBall = UNKNOWN;
	protected int rPlayMode = UNKNOWN;

	protected VisualObj self = null;
	protected VisualObj focus = null;
	protected VisualObj mtarget = null;
	protected VisualObj target = null;
	protected VisualObj ball = null;
	protected VisualObj enemyGoalT = null;
	protected VisualObj enemyGoal  = null;
	protected VisualObj enemyGoalB = null;
	protected VisualObj ourGoalT = null;
	protected VisualObj ourGoal  = null;
	protected VisualObj ourGoalB = null;
	protected Vector visFPlayers = null;
	protected Vector visEPlayers = null;

	protected int prevSeeTime = UNKNOWN;
	protected int prevFeelTime = UNKNOWN;
	protected int prevHearTime = UNKNOWN;
	protected int prevTurnTime = UNKNOWN;

	protected int assignment = P_WAIT;


	/** Konstruktor
	 * Meldet den Spieler am Server an und erledigt sonst. Aufgaben vor Beginn des Spieles
	 * @param netTraffic Netzwerkskommunikationsobjekt
	 * @param teamName Teamname
	 * @param playerNumber Spielernummer
	 * @param showGUI GUI ja oder nein? (1/0)
	 */
	public TheBrain(NetTraffic netTraffic, String teamName, int playerNumber, int showGUI)
	{
		this.netTraffic = netTraffic;
		this.playerNumber = playerNumber;
		this.teamName = teamName;

		body  = new SuperBody(netTraffic);
		switch(playerNumber)
		{
			case 1:
				netTraffic.sendePaket(new String("(init " + teamName + " (version 7.0) (goalie))"));
				break;
			default:
				netTraffic.sendePaket(new String("(init " + teamName + "(version 7.0))"));
				break;
		}
		//if Gui show GUI
		if(showGUI == 1)
		{
			this.showGUI = true;
			this.gui = new GUI(playerNumber, teamName);
		}
		showCMD = this.showGUI;
	}

	/** Bestimmt den absoluten Sichtwinkel auf dem Spielfeld ( 0 Grad Sichtrichtung auf rechtes Tor, 0 Grad Sichtrichtung auf linkes Tor vom Mittelpunkt
	 * @param lineList Vector mit allen sichtbaren Linien
	 * @return Winkel in Grad
	 */
	public double evalAbsAngle(Vector lineList)
	{
		double absAngle = OUTBOUNDS;
		double relAngle;

		if(!lineList.isEmpty())
		{
			VisualObj line = (VisualObj)lineList.firstElement();
			relAngle = line.getAngle();
			switch(line.getCode().charAt(2))
			{
				case 't':
					absAngle = relAngle >= 0 ? (360 - relAngle) : (180 - relAngle);
					break;
				case 'b':
					absAngle = relAngle >= 0 ? (180 - relAngle) : ( 0 - relAngle);
					break;
				case 'r':
					absAngle = relAngle >= 0 ? (90 - relAngle) : (270 - relAngle);
					break;
				case 'l':
					absAngle = relAngle >= 0 ? (270 - relAngle) : (90 - relAngle);
					break;
			}
		}
		if(absAngle == 360) absAngle = 0;
		return absAngle;
	}//end evalAbsAngle()

	/** Bestimmt den Geschwindigkeitesvektors eines Objektes
	 * @param oldo Objekt zum frueheren Zeitpunkt
	 * @param newo Objekt zum spaeteren Zeitpunkt
	 * @return das neuere Objekt mit berechnetem Geschwindigkeitsvektor
	 */	
	VisualObj evalDelta(VisualObj oldo, VisualObj newo)
	{
		if((oldo != null) && (newo != null)	// objekte vorhanden?
			&& (oldo.getXpos() != OUTBOUNDS) && (oldo.getYpos() != OUTBOUNDS)	// x und y vorhanden
			&& (newo.getXpos() != OUTBOUNDS) && (newo.getYpos() != OUTBOUNDS)
			&& (newo.getTime() - oldo.getTime() > 0)	// zeitliche differenz?
			&& (newo.getTime() - oldo.getTime() <= P_MAXDELTATIME))
		{
			newo.setDeltaX((newo.getXpos() - oldo.getXpos()) / (newo.getTime() - oldo.getTime()));
			newo.setDeltaY((newo.getYpos() - oldo.getYpos()) / (newo.getTime() - oldo.getTime()));
		}
		else	// nicht genug daten
		{
			newo.setDeltaX(0);
			newo.setDeltaY(0);
		}
		return newo;
	}

	/**
	 * Methode zum bestimmen der Absoluten Position von sich selbst auf dem Spielfeld
	 * @param flagList Vector mit allen sichtbaren Flaggen
	 * @param absAngle Absoluter Sichtwinkel
	 * @return Ich als VisualObj
	 */
	VisualObj evalSelf(VisualObj old, Vector flagList, double absAngle)
	{//check
		if (!flagList.isEmpty() && (absAngle >= 0))
		{
			VisualObj flag = (VisualObj)flagList.firstElement();
			double gamma = flag.getAngle() + absAngle;
			double xpos = flag.getXpos() - Math.cos(gamma / 180 * Math.PI) * flag.getDist();
			double ypos = flag.getYpos() - Math.sin(gamma / 180 * Math.PI) * flag.getDist();
			VisualObj result = new VisualObj(flag.getTime(), "self" , 0 , absAngle , xpos, ypos);
			evalDelta(old, result);
			return result;
		}
		else return null;
	}

	/** Bestimmen der absoluten Position eines zu mir relativen Objekts
	 * @param obj das zu bestimmende Objekt
	 * @param self ich-Objekt
	 * @return das berechnete Objekt
	 */
	VisualObj evalAbsPos(VisualObj obj, VisualObj self)
	{
		if((obj != null) && (self != null))
		{
			// self.getAngle ist absoluter winkel
			double gamma = obj.getAngle() + self.getAngle();
			obj.setXpos(self.getXpos() + Math.cos(gamma/180 * Math.PI) * obj.getDist());
			obj.setYpos(self.getYpos() + Math.sin(gamma/180 * Math.PI) * obj.getDist());
		}
		else
		{
			obj.setXpos(OUTBOUNDS);
			obj.setYpos(OUTBOUNDS);
		}
		return obj;
	}//end evalAbsPos()

	/**
	 * Methode zum best der abs Pos einer ganzen Liste
	 * @param list
	 * @param self
	 * @return
	 */
	Vector evalAbsList(Vector list, VisualObj self)
	{
		if(!list.isEmpty() && (self != null))
		{
			for(Enumeration e = list.elements(); e.hasMoreElements();)
			{
				evalAbsPos((VisualObj)e.nextElement(), self);
			}
		}
		return list;
	}//end evalAbsList()

	/** zum Bestimmen der zu mir relativen Entfernung eins Objektes
	 * @param obj
	 * @param self
	 * @return  */
	VisualObj evalRelDist(VisualObj obj, VisualObj self)
	{
		if((obj != null) && (self != null))
		{
			double deltaX = obj.getXpos() - self.getXpos();
			double deltaY = obj.getYpos() - self.getYpos();
			obj.setDist(Math.sqrt(deltaX * deltaX + deltaY * deltaY));
		}
		else
		{
			obj.setDist(OUTBOUNDS);
		}
		return obj;
	}//end evalRelDist()

	/** Bestimmen des zu mir relativen Winkels eines Objektes
	 */
	VisualObj evalRelAngle(VisualObj obj, VisualObj self)
	{
		if((obj != null) && (self != null))
		{
			double deltaX = obj.getXpos() - self.getXpos();
			double deltaY = obj.getYpos() - self.getYpos();
			double angle = Math.acos(deltaX / Math.sqrt(deltaX * deltaX + deltaY * deltaY));
			angle = (angle * 180 / Math.PI);
			angle = (deltaY >= 0) ? (angle) : (angle * (-1));
			angle -=  self.getAngle();
			angle = (angle > 180) ? (angle - 360) : (angle);
			angle = (angle < -180) ? (360 + angle) : (angle);
			obj.setAngle(angle);
		}
		else
		{
			obj.setAngle(OUTBOUNDS);
		}
		return obj;
	}//end evalRelAngle()

	/** Methode zum bestimmen der rel Werte eines abs Objekts
	 */
	VisualObj evalRel(VisualObj obj, VisualObj self)
	{
		if((obj != null) && (self != null))
		{
			obj = evalRelAngle(obj, self);
			obj = evalRelDist(obj, self);
		}
		return obj;
	}//end evalRelAngle()

/**
 * Methode um zu testen ob man noch in seinem Bereich ist
 *
 */
	boolean isInBoundsX(VisualObj obj)
	{
		if((obj == null)
			|| (obj.getXpos() < minX)
			|| (obj.getXpos() > maxX))
		return false;
		else return true;
	}

/**
 * Methode um zu testen ob man noch in seinem Bereich ist
 *
 */
	boolean isInBoundsY(VisualObj obj)
	{
		if((obj == null)
			|| (obj.getYpos() < minY)
			|| (obj.getYpos() > maxY))
		return false;
		else return true;
	}

/**
 * Methode um zu testen ob man noch in seinem Bereich ist
 *
 */
	boolean isInBounds(VisualObj obj)
	{
		return isInBoundsX(obj) && isInBoundsY(obj);
	}

	/**
	 * Methode um ein Event (message) abzufangen
	 * @return das angeforderte Event
	 */
	MsgEvent getEvent()
	{
		MsgEvent myEvent = null;

		while (myEvent == null) {
				// receive server message - block until one arrives
				myEvent = new MsgEvent(netTraffic.empfangePaket());
				switch(myEvent.getType())
				{
					case ERROR:
						break;
					case INIT:
						myEvent = (MsgEvent)new InitEvent(myEvent.getMessage());
						break;
					case SENSEBODY:
						myEvent = (MsgEvent)new BodyEvent(myEvent.getMessage());
						break;
					case SEE:
						myEvent = (MsgEvent)new SeeEvent(myEvent.getMessage(), teamName);
						break;
					case HEAR:
						myEvent = (MsgEvent)new HearEvent(myEvent.getMessage());
						break;
					case SERVERPARAM:
					case PLAYERPARAM:
						break;
					default:
						break;
				}
		 }
		return myEvent;
	}


	/** Eventhandler fuer die Servernachrichten
	 */
	void handleEvent()
	{

	MsgEvent currEvent = null;

	while(true){
		//liefert den nachrichtentyp, der vom server erhalten wurde
		currEvent = getEvent();
		switch(currEvent.getType())
		{
			case ERROR:
			break;
			case INIT:
				assignment = P_WAIT;
				ourSide = ((InitEvent)currEvent).getSide();
				serverPlayerNr = ((InitEvent)currEvent).getServerPlayerNr();
				initX = PINIT[playerNumber - 1][0] * ourSide;
				initY = PINIT[playerNumber - 1][1] * ourSide;
				playX = PINIT[playerNumber - 1][2] * ourSide;
				playY = PINIT[playerNumber - 1][3] * ourSide;
				if(ourSide == LEFTSIDE)
				{
					minX  = PINIT[playerNumber - 1][4];
					maxX  = PINIT[playerNumber - 1][5];
					minY  = PINIT[playerNumber - 1][6];
					maxY  = PINIT[playerNumber - 1][7];
					enemyGoalT = new VisualObj(UNKNOWN, "f g r t", OUTBOUNDS, OUTBOUNDS, FLAGSXY[23][0], FLAGSXY[23][1]);
					enemyGoal  = new VisualObj(UNKNOWN, "g r", OUTBOUNDS, OUTBOUNDS, FLAGSXY[29][0], FLAGSXY[29][1]);
					enemyGoalB = new VisualObj(UNKNOWN, "f g r b", OUTBOUNDS, OUTBOUNDS, FLAGSXY[32][0], FLAGSXY[32][1]);
					ourGoalT   = new VisualObj(UNKNOWN, "f g l t", OUTBOUNDS, OUTBOUNDS, FLAGSXY[22][0], FLAGSXY[22][1]);
					ourGoal    = new VisualObj(UNKNOWN, "g l", OUTBOUNDS, OUTBOUNDS, FLAGSXY[25][0], FLAGSXY[25][1]);
					ourGoalB   = new VisualObj(UNKNOWN, "f g l b", OUTBOUNDS, OUTBOUNDS, FLAGSXY[31][0], FLAGSXY[31][1]);
				}
				if(ourSide == RIGHTSIDE)
				{
					minX  = PINIT[playerNumber - 1][5] * ourSide;
					maxX  = PINIT[playerNumber - 1][4] * ourSide;
					minY  = PINIT[playerNumber - 1][7] * ourSide;;
					maxY  = PINIT[playerNumber - 1][6] * ourSide;;
					enemyGoalT = new VisualObj(UNKNOWN, "f g l t", OUTBOUNDS, OUTBOUNDS, FLAGSXY[22][0], FLAGSXY[22][1]);
					enemyGoal  = new VisualObj(UNKNOWN, "g l", OUTBOUNDS, OUTBOUNDS, FLAGSXY[25][0], FLAGSXY[25][1]);
					enemyGoalB = new VisualObj(UNKNOWN, "f g l b", OUTBOUNDS, OUTBOUNDS, FLAGSXY[31][0], FLAGSXY[31][1]);
					ourGoalT   = new VisualObj(UNKNOWN, "f g r t", OUTBOUNDS, OUTBOUNDS, FLAGSXY[23][0], FLAGSXY[23][1]);
					ourGoal    = new VisualObj(UNKNOWN, "g r", OUTBOUNDS, OUTBOUNDS, FLAGSXY[29][0], FLAGSXY[29][1]);
					ourGoalB   = new VisualObj(UNKNOWN, "f g r b", OUTBOUNDS, OUTBOUNDS, FLAGSXY[32][0], FLAGSXY[32][1]);
				}
				mtarget = enemyGoal;
				target = enemyGoal;
				focus = enemyGoal;
				body.moveTo(initX * ourSide, initY * ourSide);
			break;
			case SENSEBODY:
				if(currFeel != null) prevFeelTime = currFeel.getTime();
				currFeel = (BodyEvent)currEvent;
			break;
			case SEE:
				if(currSee != null) prevSeeTime = currSee.getTime();
				currSee = (SeeEvent)currEvent;
				self = evalSelf(self, currSee.getFlagList(), evalAbsAngle(currSee.getLineList()));
				if(currSee.getBall() != null)	// neuen ball gefunden?
				{
					ball = evalDelta(ball, evalAbsPos(currSee.getBall(), self));
					ballOrigin = 0;
					lastKnownBall = currSee.getTime();
					if(ball.getDist() <= P_MAXTELLDIST)	// ballposition mitteilen
					{
						body.say(teamName + " " + serverPlayerNr + " " + ball.getTime() + " " + ball.getCode() + " " + ball.getXpos() + " " + ball.getYpos() + " " + ball.getDist());
					}
				}
				else	// nein, kein ball gefunden
				{
					if((ball != null) && (currSee.getTime() - lastKnownBall > P_MAXDELTATIME))	// letzter bekannter ball zu alt?
					{
						ball = null;
						ballOrigin = OUTBOUNDS;
						lastKnownBall = UNKNOWN;
					}
					else if((ball != null) && (currSee.getTime() - ball.getTime() > 0))	// aus anderer zeit? vorherberechnung
					{
						ball.setXpos(ball.getXpos() + ball.getDeltaX() * (currSee.getTime() - ball.getTime()));
						ball.setYpos(ball.getYpos() + ball.getDeltaY() * (currSee.getTime() - ball.getTime()));
						ball.setDeltaX(ball.getDeltaX() * Math.pow(P_BALLPREDICT, currSee.getTime() - ball.getTime()));
						ball.setDeltaY(ball.getDeltaY() * Math.pow(P_BALLPREDICT, currSee.getTime() - ball.getTime()));
						ball.setTime(currSee.getTime());
						ballOrigin = OUTBOUNDS;
						evalRel(ball, self);
						if(showCMD) System.out.println(currHear.getTime() + "\tTheBrain - handleEvent - BALL VORHERBERECHNET");
					}
					else	// ball aktuell und behalten
					{
					}
				}
				enemyGoalT = evalRel(enemyGoalT, self);
				enemyGoal  = evalRel(enemyGoal , self);
				enemyGoalB = evalRel(enemyGoalB, self);
				ourGoalT = evalRel(ourGoalT, self);
				ourGoal  = evalRel(ourGoal , self);
				ourGoalB = evalRel(ourGoalB, self);
				visFPlayers = evalAbsList(currSee.getFPlayerList(), self);
				visEPlayers = evalAbsList(currSee.getEPlayerList(), self);
				play();

				//GUI
				if (gui != null)
				{
					gui.updateGUI(serverPlayerNr, currSee.getFlagList(), currSee.getLineList(), visFPlayers, visEPlayers, self, ball, enemyGoal, minX, minY, maxX, maxY, mtarget.getXpos(), mtarget.getYpos());
				}
			break;
			case HEAR:
				if(currHear != null) prevHearTime = currHear.getTime();
				currHear = (HearEvent)currEvent;
				switch(currHear.getSender())
				{
					case OTHER:	// anderer mitspieler
						StringTokenizer	tokenizer = new StringTokenizer(currHear.getHMsg(), " ");
						if(tokenizer.hasMoreTokens())	// was zu hoeren?
						{
							String hteam = tokenizer.nextToken();
							if(hteam.equals(teamName))	// aus unserem team?
							{
								int hpnr = UNKNOWN;
								int htime = UNKNOWN;
								String hcode = "";
								double hxpos = Double.NaN;
								double hypos = Double.NaN;
								double hdist = Double.NaN;
								try
								{
									hpnr = Integer.parseInt(tokenizer.nextToken());
									htime = Integer.parseInt(tokenizer.nextToken());
									hcode = tokenizer.nextToken();
									hxpos = Double.parseDouble(tokenizer.nextToken());
									hypos = Double.parseDouble(tokenizer.nextToken());
									hdist = Double.parseDouble(tokenizer.nextToken());
								}
								catch(NumberFormatException ne)
								{
									if(debug)
									{
										System.out.println("!EXCEPTION! - TheBrain - handleEvent - hear");
										ne.printStackTrace();
									}
									hdist = Double.NaN;
								}
								if(debug) System.out.println(currHear.getTime() + "\tTheBrain - handleEvent - NEUEN BALL GEHOERT: " + htime + " | " + Double.toString(hxpos) + "/" + Double.toString(hypos) + "/" + Double.toString(hdist));
								if(hcode.equals("b")	// ist es der ball?
									&& (hdist != Double.NaN)	// fehler bei der konvertierung?
									&& (hdist < ballOrigin)	// ist er aus einer naeheren pos?
									&& (hxpos != Double.NaN)	// fehler bei der konvertierung?
									&& (hypos != Double.NaN)	// fehler bei der konvertierung?
									&& (htime >= lastKnownBall))	// neuer als letzter bekannter?
								{
									VisualObj hball = new VisualObj(htime, hcode);
									hball.setXpos(hxpos);
									hball.setYpos(hypos);
									ball = evalDelta(ball, evalRel(hball, self));
									ballOrigin = hdist;
									lastKnownBall = htime;
									if(showCMD) System.out.println(currHear.getTime() + "\tTheBrain - handleEvent - NEUEN BALL ANGENOMMEN");
								}
							}
						}
					break;
					case REFEREE:
						rPlayMode = currHear.getCmd();
						switch(currHear.getCmd())
						{
							case BEFORE_KICK_OFF        :
							case HALFTIME               :
							case TIME_EXTENDED          :
								body.moveTo(initX * ourSide, initY * ourSide);
								status = START;
							break;
							case GOAL_RIGHT             :
							case GOAL_LEFT              :
								body.moveTo(initX * ourSide, initY * ourSide);
								status = START;
							break;
							case GOALIE_CATCH_BALL_RIGHT:
							case GOALIE_CATCH_BALL_LEFT :
							case CORNER_KICK_RIGHT      :
							case CORNER_KICK_LEFT       :
							case DROP_BALL              :
							case FREE_KICK_RIGHT        :
							case FREE_KICK_LEFT         :
							case FOUL_RIGHT             :
							case FOUL_LEFT              :
							case GOAL_KICK_RIGHT        :
							case GOAL_KICK_LEFT         :
							case KICK_IN_RIGHT          :
							case KICK_IN_LEFT           :
							case KICK_OFF_RIGHT         :
							case KICK_OFF_LEFT          :
							case OFFSIDE_RIGHT          :
							case OFFSIDE_LEFT           :
							case PLAY_ON                :
								status = PLAYING;
							break;
							case TIME_OVER              :
							case TIME_UP                :
							case TIME_UP_WITHOUT_A_TEAM :
								System.out.println("\nNormal Exit\n");
								System.exit(0);
							break;
						}
					break;
				}
			break;
			case SERVERPARAM:
			break;
			case PLAYERPARAM:
			break;

			} //end switch
		} //end while
	} //end handleEvent()

	/** Verkuerzt oder verlaengert Richtungsvektoren
	 * @param obj Startpunkt
	 * @param target Zielpunkt
	 * @param vlength Vektorlaenge(positiv oder negativ)
	 * @return Emdpunkt des neuen Vektors vom Startpunkt aus gesehen
	 */	
	VisualObj getLineUp(VisualObj obj, VisualObj target, double vlength)
	{
		if((obj != null)
			&& (target != null)
			&& (obj.getXpos() != OUTBOUNDS)
			&& (obj.getYpos() != OUTBOUNDS)
			&& (target.getXpos() != OUTBOUNDS)
			&& (target.getYpos() != OUTBOUNDS))
		{
			VisualObj result = new VisualObj(obj.getTime(), obj.getCode());
			double dx = target.getXpos() - obj.getXpos();
			double dy = target.getYpos() - obj.getYpos();
			double dc = Math.sqrt(dx * dx + dy * dy);
			dx = dx / dc * vlength;
			dy = dy / dc * vlength;
			result.setXpos(obj.getXpos() + dx);
			result.setYpos(obj.getYpos() + dy);
			result = evalRel(result, self);
			return result;
		}
		else return null;
	}
	
	/** Vorherprojektion des Balls
	 * @param ball Ball
	 * @param self Ich
	 * @return vorherprojezierter Ball
	 */	
	VisualObj predictBall(VisualObj ball, VisualObj self)
	{
		if(ball != null)
		{
//			return evalRel(new VisualObj(ball.getTime(), ball.getCode(), OUTBOUNDS, OUTBOUNDS, ball.getXpos() + ball.getDeltaX() * P_BALLPREDF, ball.getYpos() + ball.getDeltaY() * P_BALLPREDF), self);
			return ball;
		}
		else return null;
	}


	/** Berechnet den rel. Drehwinkel zum Positionieren zwischen 2 Objekten
	 * @param a rel. Winkel von Objekt a
	 * @param b rel. Winkel von Objekt b
	 * @return Winkel um zwischen den Objekten durchzuschauen
	 */	
	double getLineUpAngle(double a, double b)
	{
		double angle = (a + b) / 2;
		angle = (angle > 180) ? (angle - 360) : (angle);
		angle = (angle < -180) ? (360 + angle) : (angle);
		return angle;
	}

	/** Winkel zwischen 2 rel Objekten
	 * @param b rel. Winkel von Objekt b
	 * @param a rel. Winkel von Objekt a
	 * @return Winkel zwischen den beiden Objekten
	 */	
	double getDeltaAngle(double a, double b)
	{
		double angle = Math.abs(b - a);
		angle = (angle > 180) ? (angle - 360) : (angle);
		return angle;
	}

	/** bestimmen der Kickstaerke in Abhaengigkeit der Entfernung zum Ziel
	 * @param target Ziel
	 * @return Kickstaerke
	 */	
	double getKickStrength(VisualObj target)
	{
		if(target != null)
		{
			return 100;
		}
		else return 0;
	}

	/** bestimmen der Laufstaerke in Abhaengigkeit der Entfernung zum Ziel
	 * @param target Ziel
	 * @return Laufstaerke
	 */	
	double getDashStrength(VisualObj target)
	{
		if(target != null)
		{
			return 100;
		}
		else return 0;
	}

	/** Wo soll ich hin?
	 * @param obj Referenzobjekt
	 * @return Laufziel
	 */	
	VisualObj getFocusWait(VisualObj obj)
	{
		if(obj != null)
		{
			return obj;
		}
		else return null;
	}

	/** Ermittelt das beste Angriffs bzw Passziel
	 * @return Angriffs oder Passziel
	 */	
	VisualObj getBestTarget()
	{
		return enemyGoal;
	}

	/** entscheidet ueber das Verhalten des Spielers
	 * @return Zustandsvariable im Flagformat (siehe Konstanten.java)
	 */	
	int getDesicion()
	{
		if((self != null) && (ball != null))	// eigene Pos und ball bekannt?
		{
			target = getBestTarget();
			mtarget = ball;
			assignment = P_ATTACKTO + P_MOVETO;
		}
		else
			assignment = P_WAIT;
		return assignment;
	}

	/** Setzt die Entscheidung von getDesicion
	 */
	void play()
	{
		if(status == START)
		{
		}
		else if(status == PLAYING)
		{
			assignment = getDesicion();
			if(assignment == P_WAIT)	// nichts zu tun!
			{
				body.turn(P_SEEKANGLE);
				if(showCMD) System.out.println(currSee.getTime() + "\tTheBrain - play - SUCHEN...");
			}
			if((assignment & P_ATTACKTO) > 0)	// ball schiessen
			{
				if((ball != null) && (target != null))	// ball und ziel bekannt?
				{
					if(ball.getDist() <= P_MAXBALLSHOOTDIST)	// ball in schussreichweite?
					{
						double deltangle = getDeltaAngle(ball.getAngle(), target.getAngle());
						double lineangle = getLineUpAngle(ball.getAngle(), target.getAngle());
						if(Math.abs(lineangle) <= P_MAXOFFTARGETSHOOTANGLE)	// winkelabst. zwischen ball und toor?
						{
							body.kick(getKickStrength(target), target.getAngle());
							if(showCMD) System.out.println(currSee.getTime() + "\tTheBrain - play - SCHUSS AUSGEFUEHRT");
						}
						else if((Math.abs(lineangle) > P_MAXOFFTARGETSHOOTANGLE)	// in reichweite aber nicht nicht im schusswinkel
							&& (prevTurnTime != currSee.getTime()))	// schon am drehen?
						{
							body.turn(lineangle);
							prevTurnTime = ball.getTime();
							if(showCMD) System.out.println(currSee.getTime() + "\tTheBrain - play - SCHUSSRICHTUNG KORREGIERT");
						}
					}
					else	// nicht in reichweite
					{
					}
				}
			}
			if((assignment & P_DEFENSETO) > 0)	// ball passen
			{
			}
			if((assignment & P_CATCH) > 0)	// ball fangen
			{
				if(ball != null)	// ball bekannt?
				{
					if((ball.getDist() <= P_MAXBALLCATCHDIST)	// ball in reichweite?
						&& (Math.abs(ball.getAngle()) <= P_MAXBALLCATCHANGLE))	// und winkel?
					{
						body.gCatch(ball.getAngle());
						if(showCMD) System.out.println(currSee.getTime() + "\tTheBrain - play - BALL GEFANGEN");
					}
				}
			}
			if((assignment & P_MOVETO) > 0)	// zum target laufen?
			{
				if((mtarget != null)
					&& (Math.abs(mtarget.getAngle()) != OUTBOUNDS)
					&& (Math.abs(mtarget.getDist()) != OUTBOUNDS))	// taget bekannt?
				{
					if((mtarget.getDist() >= P_MAXMTARGETDIST)	// target ausser reichweite
						&& (Math.abs(mtarget.getAngle()) >= P_MAXMTARGETANGLE))	//focus bekannt blick aber nicht auf sicht
					{
						if(prevTurnTime != currSee.getTime())
						{
							body.turn(mtarget.getAngle());
							prevTurnTime = mtarget.getTime();
							if(showCMD) System.out.println(currSee.getTime() + "\tTheBrain - play - ZUM ZIEL GEDREHT");
						}
					}
					else if(mtarget.getDist() >= P_MAXMTARGETDIST)	// target auf sicht
					{
						body.dash(getDashStrength(mtarget));
						if(showCMD) System.out.println(currSee.getTime() + "\tTheBrain - play - ZUM ZIEL GLEAUFEN");
					}
					else	// target in reichweite
					{
					}
				}
				else // target unbekannt
				{
				}
			}
			if((assignment & P_FOCUSON) > 0)	// im blick behalten
			{
				if((focus != null) && (Math.abs(focus.getAngle()) != OUTBOUNDS))	// focus bekannt?
				{
					if(Math.abs(focus.getAngle()) > P_MAXFOCUSANGLE)	//focus bekannt blick aber nicht auf sicht
					{
						if(prevTurnTime != currSee.getTime())
						{
							body.turn(focus.getAngle());
							prevTurnTime = focus.getTime();
							if(showCMD) System.out.println(currSee.getTime() + "\tTheBrain - play - FOCUS KORREGIERT");
						}
					}
					else	// focus auf sicht
					{
					}
				}
				else	// focus nicht bekannt
				{
				}
			}
			if((assignment & P_MOVEXY) > 0)	// neue Pos?
			{
				
			}
		}
	}

} //end class TheBrain
